from django.db import migrations, models


def code_make_labels_unique(apps, schema_editor):
    Source = apps.get_model(app_label='sources', model_name='Source')

    for source in Source.objects.using(alias=schema_editor.connection.alias).all():
        # Look for sources with the same label.
        queryset_duplicates = Source.objects.using(
            alias=schema_editor.connection.alias
        ).filter(label=source.label).exclude(pk=source.pk)
        if queryset_duplicates.exists():
            # If a duplicate is found, append the id to the original source
            # label.
            source.label = '{}__{}'.format(source.label, source.pk)
            source.save()


def code_make_labels_unique_reverse(apps, schema_editor):
    Source = apps.get_model(app_label='sources', model_name='Source')

    for source in Source.objects.using(alias=schema_editor.connection.alias).all():
        if source.label.endswith('__{}'.format(source.pk)):
            source.label = source.label.replace(
                '__{}'.format(source.pk), ''
            )
            source.save()


class Migration(migrations.Migration):
    dependencies = [
        ('sources', '0018_auto_20180608_0057')
    ]

    operations = [
        migrations.RunPython(
            code=code_make_labels_unique,
            reverse_code=code_make_labels_unique_reverse
        ),
        migrations.AlterField(
            model_name='source',
            name='label',
            field=models.CharField(
                db_index=True, max_length=64, unique=True,
                verbose_name='Label'
            )
        )
    ]



from django.utils.translation import gettext_lazy as _

from mayan.apps.permissions.classes import PermissionNamespace

namespace = PermissionNamespace(
    label=_(message='Metadata'), name='metadata'
)

permission_document_metadata_add = namespace.add_permission(
    label=_(message='Add metadata to a document'), name='metadata_document_add'
)
permission_document_metadata_edit = namespace.add_permission(
    label=_(message='Edit a document\'s metadata'), name='metadata_document_edit'
)
permission_document_metadata_remove = namespace.add_permission(
    label=_(message='Remove metadata from a document'),
    name='metadata_document_remove'
)
permission_document_metadata_view = namespace.add_permission(
    label=_(message='View metadata from a document'), name='metadata_document_view'
)

metadata_type_namespace = PermissionNamespace(
    label=_(message='Metadata types'), name='metadata_setup'
)

permission_metadata_type_edit = metadata_type_namespace.add_permission(
    label=_(message='Edit metadata types'), name='metadata_type_edit'
)
permission_metadata_type_create = metadata_type_namespace.add_permission(
    label=_(message='Create new metadata types'), name='metadata_type_create'
)
permission_metadata_type_delete = metadata_type_namespace.add_permission(
    label=_(message='Delete metadata types'), name='metadata_type_delete'
)
permission_metadata_type_view = metadata_type_namespace.add_permission(
    label=_(message='View metadata types'), name='metadata_type_view'
)



from django.utils.translation import gettext_lazy as _

from mayan.apps.smart_settings.settings import setting_cluster

from .literals import DEFAULT_EVENTS_DISABLE_ASYNCHRONOUS_MODE

setting_namespace = setting_cluster.do_namespace_add(
    label=_(message='Events'), name='events'
)

setting_disable_asynchronous_mode = setting_namespace.do_setting_add(
    default=DEFAULT_EVENTS_DISABLE_ASYNCHRONOUS_MODE,
    global_name='EVENTS_DISABLE_ASYNCHRONOUS_MODE',
    help_text=_(
        message='Disables asynchronous events mode. All events will be committed '
        'in the same process that triggers them. This was the behavior '
        'prior to version 4.5.'
    )
)



from django.contrib.humanize.templatetags.humanize import intcomma
from django.template import loader

from .icons import icon_dashboard_link_icon


class Dashboard:
    _registry = {}

    @classmethod
    def get(cls, name):
        return cls._registry[name]

    @classmethod
    def get_all(cls):
        return sorted(
            cls._registry.values(), key=lambda x: x.label
        )

    def __init__(self, name, label):
        self.name = name
        self.label = label
        self.widgets = {}
        self.removed_widgets = []
        self.__class__._registry[name] = self

    def add_widget(self, widget, order=0):
        self.widgets[widget] = {'widget': widget, 'order': order}

    def get_widgets(self):
        """
        Returns a list of widgets sorted by their 'order'.
        If two or more widgets have the same 'order', sort by label.
        """
        return map(
            lambda x: x['widget'],
            filter(
                lambda x: x['widget'] not in self.removed_widgets,
                sorted(
                    self.widgets.values(),
                    key=lambda x: (
                        x['order'], x['widget'].label
                    )
                )
            )
        )

    def remove_widget(self, widget):
        self.removed_widgets.append(widget)

    def render(self, request):
        rendered_widgets = [
            widget().render(request=request) for widget in self.get_widgets()
        ]

        return loader.render_to_string(
            template_name='dashboards/dashboard.html', context={
                'dashboard': self, 'widgets': rendered_widgets
            }
        )


class BaseDashboardWidget:
    _registry = {}
    context = {}
    template_name = None

    @classmethod
    def get(cls, name):
        return cls._registry[name]

    @classmethod
    def get_all(cls):
        return cls._registry.items()

    @classmethod
    def register(cls, klass):
        cls._registry[klass.name] = klass

    def get_base_context(self):
        raise NotImplementedError

    def get_context(self):
        return {}

    def render(self, request):
        self.request = request
        context = self.get_base_context()
        context.update(
            self.get_context()
        )
        context.update(
            {'request': request}
        )

        if self.template_name:
            return loader.render_to_string(
                context=context, template_name=self.template_name
            )


class DashboardWidgetNumeric(BaseDashboardWidget):
    count = 0
    icon = None
    label = None
    link = None
    link_icon = icon_dashboard_link_icon
    template_name = 'dashboards/numeric_widget.html'

    def get_base_context(self):
        return {
            'count': intcomma(
                value=self.get_count()
            ),
            'count_raw': self.count,
            'icon': self.icon,
            'label': self.label,
            'link': self.link,
            'link_icon': self.link_icon
        }


class DashboardWidgetList(BaseDashboardWidget):
    columns = None
    icon = None
    label = None
    link = None
    link_icon = icon_dashboard_link_icon
    object_list_length = 10
    template_name = 'dashboards/widget_list.html'

    def get_base_context(self):
        return {
            'columns': self.columns or (),
            'object_list': self.get_object_list()[:self.object_list_length],
            'icon': self.icon,
            'label': self.label,
            'link': self.link,
            'link_icon': self.link_icon
        }



from django.utils.translation import gettext_lazy as _

from mayan.apps.sources.source_backend_actions.interface_arguments import (
    SourceBackendActionInterfaceArgument
)

argument_expand = SourceBackendActionInterfaceArgument(
    default=False, help_text=_(
        message='Controls whether or not the uploaded file will be uncompressed '
        'and processed and individual files.'
    ), required=False
)



from mayan.apps.dependencies.classes import PythonDependency

PythonDependency(
    module=__name__, name='gevent', version_string='==24.2.1'
)
PythonDependency(
    module=__name__, name='greenlet', version_string='==3.0.3'
)
PythonDependency(
    module=__name__, name='gunicorn', version_string='==22.0.0'
)
PythonDependency(
    module=__name__, name='sentry-sdk', version_string='==2.35.0'
)
PythonDependency(
    module=__name__, name='whitenoise', version_string='==6.7.0'
)



from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0051_documentpage_enabled')
    ]

    operations = [
        migrations.AlterField(
            model_name='documentpage',
            name='document_version',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='version_pages', to='documents.DocumentVersion',
                verbose_name='Document version'
            )
        )
    ]



from django.utils.translation import gettext_lazy as _

from mayan.apps.permissions.classes import PermissionNamespace

namespace = PermissionNamespace(
    label=_(message='Access control lists'), name='acls'
)

permission_acl_edit = namespace.add_permission(
    label=_(message='Edit ACLs'), name='acl_edit'
)
permission_acl_view = namespace.add_permission(
    label=_(message='View ACLs'), name='acl_view'
)
